/*
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <cheerp/client.h>

auto webMain() noexcept -> auto
{
	static constexpr char secretStr[] = "saRpmZ6mMgZpmcojUkvkyGEQGez9YkWoXZfJdRc9ZmmJrCzfM8JksVxQfQK8uBBs";

	// Get the input element.
	const auto pInput = static_cast<::client::HTMLInputElement*>(::client::document.getElementById("fileDialog"));

	// Add an event listener to the input element.
	pInput->addEventListener("change", ::cheerp::Callback([pInput]()
		{
			// Get the file reader.
			const auto pReader = new ::client::FileReader();

			// Read the file as an ArrayBuffer.
			pReader->readAsArrayBuffer((*pInput->get_files())[0]);

			// Set the callback for fully loading the file.
			pReader->addEventListener("load", ::cheerp::Callback([pInput, pReader]()
				{
					// Get a data view for the file.
					const auto pDataView = new ::client::DataView{ pReader->get_result<::client::ArrayBuffer*>() };
					
					// Decrypt the scrambled JSON file by XORing each byte using the secret string (excluding its NULL delimiter).
					for (auto iChar = static_cast<int>(pDataView->get_byteLength()); iChar--;)
						pDataView->setInt8(iChar, static_cast<char>(pDataView->getInt8(iChar)) ^ secretStr[iChar % static_cast<char>(sizeof(secretStr) - 1)]);

					// Download the decrypted file.
					{
						// Create the anchor download element.
						const auto pAnchor = static_cast<::client::HTMLAnchorElement*>(::client::document.createElement("a"));

						// Set the download link.
						pAnchor->set_href(::client::URL.createObjectURL(new ::client::Blob(new ::client::Array(pDataView))));

						// Set the download's default name.
						pAnchor->set_download("tspud-savedata.txt");

						// Start the download.
						pAnchor->click();
					}

					// Clear the input to allow reuse.
					pInput->set_value("");
				}));
		}));
}
